import * as React from 'react';
import {observer} from "mobx-react-lite";
import {asyncReactEffect, FormProps, ModelForm, ModelInput} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {TabsContainer} from "../../components/generic/TabsContainer";
import {
    Dcs2restConfigModel,
    InterfaceConfigResp,
    getDefaultInterfaceConfigs,
    InterfaceConfigKeys,
    interfaceConfigKeys
} from "../../stores/Dcs2restConfigStore";
import {Box, LinearProgress} from "@material-ui/core";
import {MachineStateInterfaceConfigForm} from '../../components/dcs2rest_configs/interface_configs/MachineStateInterfaceConfigForm';
import {ErrorInterfaceConfigForm} from "../../components/dcs2rest_configs/interface_configs/ErrorInterfaceConfigForm";
import {JobStateInterfaceConfigForm} from "../../components/dcs2rest_configs/interface_configs/JobStateInterfaceConfigForm";
import {ProcessParameterInterfaceConfigForm} from "../../components/dcs2rest_configs/interface_configs/ProcessParameterInterfaceConfigForm";
import {MaintenanceInterfaceConfigForm} from "../../components/dcs2rest_configs/interface_configs/MaintenanceInterfaceConfigForm";
import {CalibrationInterfaceConfigForm} from "../../components/dcs2rest_configs/interface_configs/CalibrationInterfaceConfigForm";
import {EventInterfaceConfigForm} from "../../components/dcs2rest_configs/interface_configs/EventInterfaceConfigForm";
import {useState} from "react";

type Dcs2restConfigFormProps = {
    model: Dcs2restConfigModel
} & Omit<FormProps, 'model'>

export const Dcs2restConfigForm: React.FunctionComponent<Dcs2restConfigFormProps> = observer(({model}) => {
    const [defaultInterface, setDefaultInterface] = useState<undefined|InterfaceConfigResp>(undefined);

    React.useEffect(asyncReactEffect(async () => {
            await model.getInterfaceConfigExamples();
            return await getDefaultInterfaceConfigs();
        },
        (data: InterfaceConfigResp) => {
            setDefaultInterface(data);
        }
    ), []);

    React.useEffect(() => {
        if(!defaultInterface) { return; }
        if (model.isNew) {
            // Seed models
            const defaultInterfaceConfigs = defaultInterface;
            injectTempId(defaultInterfaceConfigs);
            model.viewModel.interface_configs = defaultInterfaceConfigs;
        }
    }, [model, defaultInterface]);

    const injectTempId = React.useCallback((interfaceConfigs: InterfaceConfigResp) => {
        let temp_id = 0;

        for (const key of interfaceConfigKeys) {
            const typedConfig = interfaceConfigs[key as InterfaceConfigKeys];
            typedConfig.forEach((tConfig) => {
                tConfig.id = temp_id;
                temp_id++;
            }) // temporary ID for fetching, filtered out in back-end
        }

    }, []);

    if (!model.viewModel.interface_configs || !defaultInterface) {
        return <LinearProgress color={'secondary'}/>;
    }

    return (
        <ModelForm model={model} suppressImportExport suppressMakeCopy>
            <ModelInput name={'name'}/>
            <Box mt={2}>
                <TabsContainer
                    tabs={
                        [
                            {
                                label: t`Machine State`,
                                component: <MachineStateInterfaceConfigForm/>,
                                disabled: false,
                                defaultTab: true
                            },
                            {
                                label: t`Error`,
                                component: <ErrorInterfaceConfigForm/>,
                                disabled: false,
                                defaultTab: false
                            },
                            {
                                label: t`Job State`,
                                component: <JobStateInterfaceConfigForm/>,
                                disabled: false,
                                defaultTab: false
                            },
                            {
                                label: t`Process Parameters`,
                                component: <ProcessParameterInterfaceConfigForm/>,
                                disabled: false,
                                defaultTab: false
                            },
                            {
                                label: t`Maintenance`,
                                component: <MaintenanceInterfaceConfigForm/>,
                                disabled: false,
                                defaultTab: false
                            },
                            {
                                label: t`Calibration`,
                                component: <CalibrationInterfaceConfigForm/>,
                                disabled: false,
                                defaultTab: false
                            },
                            {
                                label: t`Event`,
                                component: <EventInterfaceConfigForm/>,
                                disabled: false,
                                defaultTab: false
                            }
                        ]
                    }
                    variant="scrollable"
                />
            </Box>
        </ModelForm>
    );
});
