import {attribute, DataStore, Model, model, Types} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro"
import {action, override} from "mobx";

export const interfaceConfigKeys = ['machine_state', 'error', 'job_state', 'process_parameter', 'maintenance', 'calibration', 'event'] as const;
export type InterfaceConfigKeys = typeof interfaceConfigKeys[number];

export type InterfaceConfigResp = Readonly<{
    [key in InterfaceConfigKeys]: InterfaceConfigData;
}>;

export type InterfaceConfigData = ReadonlyArray<InterfaceConfigDatum>;

type InterfaceConfigDatum = {
    id: number | null;
    x_status: number;
    data: any; // object containing key-pair values that depend on the config
};

@model({
    transportLayer: "/api/dcs2rest_configs",
    permissionTarget: "Dcs2restConfig",
    routingInfo: {
        create: '/settings/dcs2rest/interfaces/create',
        read: '/settings/dcs2rest/interfaces/:id/edit',
        update: '/settings/dcs2rest/interfaces/:id/edit'
    }
})
export class Dcs2restConfigModel extends Model {
    types: Types =  {
        interface_configs: 'json'
    }

    @override
    get isUserDeletable() {
        return this.connected_machine_ids.length === 0;
    }

    @attribute({
        label: t`Name`,
        validation: yup.string()
    })
    name: string = '';

    @attribute({})
    connected_machine_ids: number[] = []

    @attribute({
        label: t`Minimum`,
        validation: yup.number().min(0)
    })
    machine_state_min: number = 200;

    @attribute({
        label: t`Maximum`,
        validation: yup.number().min(0)
    })
    machine_state_max: number = 299;

    @attribute({
        label: t`Minimum`,
        validation: yup.number().min(0)
    })
    error_min: number = 400;

    @attribute({
        label: t`Maximum`,
        validation: yup.number().min(0)
    })
    error_max: number = 499;

    @attribute({
        label: t`Minimum`,
        validation: yup.number().min(0)
    })
    job_state_min: number = 300;

    @attribute({
        label: t`Maximum`,
        validation: yup.number().min(0)
    })
    job_state_max: number = 399;

    @attribute({
        label: t`Minimum`,
        validation: yup.number().min(0).nullable()
    })
    process_parameter_min: number = 0;

    @attribute({
        label: t`Maximum`,
        validation: yup.number().min(0).nullable()
    })
    process_parameter_max: number = 0;

    @attribute({
        label: t`Minimum`,
        validation: yup.number().min(0)
    })
    maintenance_min: number = 500;

    @attribute({
        label: t`Maximum`,
        validation: yup.number().min(0)
    })
    maintenance_max: number = 599;

    @attribute({
        label: t`Minimum`,
        validation: yup.number().min(0)
    })
    calibration_min: number = 600;

    @attribute({
        label: t`Maximum`,
        validation: yup.number().min(0)
    })
    calibration_max: number = 699;

    @attribute({
        label: t`Minimum`,
        validation: yup.number().min(0)
    })
    event_min: number = 700;

    @attribute({
        label: t`Maximum`,
        validation: yup.number().min(0)
    })
    event_max: number = 799;

    @attribute()
    interface_examples: Record<string, any> | null  = null

    @attribute()
    interface_configs: InterfaceConfigResp | null = null;

    @attribute()
    created_at: Date = new Date();

    @attribute()
    updated_at: Date = new Date();

    constructor() {
        super();
        super.setup();
    }

    // Making this fetcher on the model itself because we need the result at multiple spots, so using the model to store the result.
    @action
    async getInterfaceConfigExamples() {
        if(this.viewModel.interface_examples) { return; }
        const resp = await Dcs2restConfigStore.transportLayer.fetch("/interface_config_examples_json", {method: "get"})
        this.viewModel.interface_examples = await resp.json();
    }
}

export const getDefaultInterfaceConfigs = async () => {
    const response = await Dcs2restConfigStore.transportLayer.fetch("/default_config_json", {method: "get"})
    return await response.json();
};

export const Dcs2restConfigStore = Dcs2restConfigModel.store as DataStore<Dcs2restConfigModel>;
