import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {action} from "mobx";

@model({
    transportLayer: "/api/dcs_configs",
    permissionTarget: "DcsConfig",
    routingInfo: {
        create: '/dcs_connections/create',
        read: '/dcs_connections/:id/edit',
        update: '/dcs_connections/:id/edit'
    }
})
export class DcsConfigModel extends Model {
    @attribute()
    id: number = 0;

    @attribute({
        label: t`System Name`,
        validation: yup.string().min(3).nullable(false)
    })
    name: string = '';

    @attribute({
        label: t`Short Description`,
        validation: yup.string().min(3).max(20).nullable(false)
    })
    description: string = '';

    @attribute({
        label: t`IP Address`,
        validation: yup.string().nullable(false)
            .test('valid-ip', t`IP Address must be a valid IP address.`, (val) => {
                const ipRegex = /^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$/i;
                return ipRegex.test(val);
            })
    })
    ip: string = '';

    @attribute({
        label: t`Port`,
        validation: yup.number().positive()
    })
    port: number = 0;

    @attribute({
        label: t`Device Type`,
        validation: yup.string().min(3).nullable(false)
    })
    device_type: string = '';

    @attribute({
        label: t`Retry attempts on failed connections`,
        validation: yup.number().min(0)
    })
    retry_on_fail_connection: number = 0;

    @attribute({
        label: t`Connection Timeout (seconds)`,
        validation: yup.number().min(3)
    })
    connection_timeout: number = 5;

    @attribute({
        label: t`Maximum Allowed Connection Attempts`,
        validation: yup.number().nullable(true)
    })
    max_open_connections: number | null = null;

    @attribute({
        label: t`Delay on Socket Open (seconds)`,
        validation: yup.number().nullable(true)
    })
    delay_on_socket_open: number | null = null;

    @attribute({
        label: t`Hold Connection Open`,
        validation: yup.boolean().required()
    })
    hold_connection_open: boolean = true;

    @attribute({
        label: t`Save Call Trace`,
        validation: yup.boolean().required()
    })
    keep_call_trace: boolean = true;

    @action
    async performJobCall(job: string, tags: string[]){
        const resp = await fetch(`${DcsConfigStore.transportLayer.base}/${this.id}/job_call`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                job_call: {
                    job: job,
                    tags: tags
                }
            })
        });

        return await resp.json();
    }

    constructor() {
        super();
        super.setup();
    }
}

export const DcsConfigStore = DcsConfigModel.store as DataStore<DcsConfigModel>;
